from os.path import join, isfile, isdir
from os import listdir
import os
os.environ["CUDA_VISIBLE_DEVICES"] = "0"
from argparse import ArgumentParser

import detectron2
from detectron2.utils.logger import setup_logger
setup_logger()

import numpy as np
import cv2
from ColorTransferLib.Utils.Helper import get_cache_dir

# import some common detectron2 utilities
from detectron2 import model_zoo
from detectron2.engine import DefaultPredictor
from detectron2.config import get_cfg

import torch
from tqdm import tqdm

def predict_bbox(src_img):
    cfg = get_cfg()
    cfg.merge_from_file(model_zoo.get_config_file("COCO-InstanceSegmentation/mask_rcnn_X_101_32x8d_FPN_3x.yaml"))
    cfg.MODEL.ROI_HEADS.SCORE_THRESH_TEST = 0.7
    cfg.MODEL.WEIGHTS = model_zoo.get_checkpoint_url("COCO-InstanceSegmentation/mask_rcnn_X_101_32x8d_FPN_3x.yaml")
    predictor = DefaultPredictor(cfg)

    # parser = ArgumentParser()
    # parser.add_argument("--test_img_dir", type=str, default='example', help='testing images folder')
    # parser.add_argument('--filter_no_obj', action='store_true')
    # args = parser.parse_args()

    # input_dir = args.test_img_dir
    # image_list = [f for f in listdir(input_dir) if isfile(join(input_dir, f))]
    # output_npz_dir = "{0}_bbox".format(input_dir)
    # if os.path.isdir(output_npz_dir) is False:
    #     print('Create path: {0}'.format(output_npz_dir))
    #     os.makedirs(output_npz_dir)

    cache_dir = get_cache_dir()
    method_cache_dir = os.path.join(cache_dir, "IIC")
    os.makedirs(method_cache_dir, exist_ok=True)



    # for image_path in tqdm(image_list):

    # img = cv2.imread(join(input_dir, image_path))
    img = src_img
    # lab_image = cv2.cvtColor(img, cv2.COLOR_BGR2LAB)
    lab_image = cv2.cvtColor(img, cv2.COLOR_RGB2LAB)
    l_channel, a_channel, b_channel = cv2.split(lab_image)
    l_stack = np.stack([l_channel, l_channel, l_channel], axis=2)
    outputs = predictor(l_stack)

    save_path = cache_dir + '/IIC/' + 'bbox.npz'
    pred_bbox = outputs["instances"].pred_boxes.to(torch.device('cpu')).tensor.numpy()
    pred_scores = outputs["instances"].scores.cpu().data.numpy()
    # if args.filter_no_obj is True and pred_bbox.shape[0] == 0:
    #     print('delete {0}'.format(image_path))
    #     os.remove(join(input_dir, image_path))
    #     return
    np.savez(save_path, bbox = pred_bbox, scores = pred_scores)    
    
    return (pred_bbox, pred_scores)
